@@ -90,8 +90,8 @@ namespace Beagle.Daemon.FileSystemQueryable {
                                 Logger.Log.Debug ("Starting Inotify Backend");
                                 event_backend = new InotifyBackend ();
                         } else {
-                                Logger.Log.Debug ("Starting FileSystemWatcher Backend");
-                                event_backend = new FileSystemWatcherBackend ();
+                                Logger.Log.Debug ("Creating null file event backend");
+				event_backend = new NullFileEventBackend ();
                         }
 
 			tree_crawl_task = new TreeCrawlTask (new TreeCrawlTask.Handler (AddDirectory));
@@ -722,6 +722,10 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			FileAttributes attr;
 			attr = FileAttributesStore.Read (dir.FullName);
 
+			// Don't mark ourselves; let the crawler redo us
+			if (attr == null)
+				return;
+
 			// We don't have to be super-careful about this since
 			// we only use the FileAttributes mtime on a directory
 			// to determine its initial state, not whether or not
@@ -761,7 +765,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			bool path_is_registered = true;
 
 			if (dir == null) {
-				dir = GetDirectoryModelByPath (Path.GetDirectoryName (path));
+				dir = GetDirectoryModelByPath (FileSystem.GetDirectoryNameRootOk (path));
 				path_is_registered = false;
 
 				if (dir == null) {
