@@ -32,8 +32,6 @@ using System.IO;
 using Beagle.Util;
 using Beagle.Daemon;
 
-using Mono.Posix;
-
 namespace Beagle.Daemon.KMailQueryable {
 	
 	/**
@@ -244,6 +242,14 @@ namespace Beagle.Daemon.KMailQueryable {
 			Queue pending = new Queue ();
 			pending.Enqueue (mail_root);
 			folder_directories.Add (mail_root);
+			// add inotify watch to root folder
+			if (Inotify.Enabled)
+				Inotify.Subscribe (mail_root, OnInotifyEvent,
+					Inotify.EventType.Create
+					| Inotify.EventType.Delete
+					| Inotify.EventType.MovedFrom
+					| Inotify.EventType.MovedTo
+					| Inotify.EventType.Modify);
 
 			while (pending.Count > 0) {
 
@@ -384,7 +390,7 @@ namespace Beagle.Daemon.KMailQueryable {
 			// is deleted, all the messages in that file can be deleted
 			indexable.ParentUri = UriFu.PathToFileUri (file_name);
 
-			indexable.Timestamp = message.Date;
+			indexable.Timestamp = message.Date.ToUniversalTime ();
 			indexable.HitType = "MailMessage";
 			indexable.MimeType = "message/rfc822";
 			indexable.CacheContent = false;
@@ -397,37 +403,36 @@ namespace Beagle.Daemon.KMailQueryable {
 
 			addrs = message.GetRecipients (GMime.Message.RecipientType.To);
 			foreach (GMime.InternetAddress ia in addrs) {
-				if (folder_name == Queryable.SentMailFolderName)
+				if (folder_name == Queryable.SentMailFolderName && ia.AddressType != GMime.InternetAddressType.Group)
 					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
 			}
 			addrs.Dispose ();
 
 			addrs = message.GetRecipients (GMime.Message.RecipientType.Cc);
 			foreach (GMime.InternetAddress ia in addrs) {
-				if (folder_name == Queryable.SentMailFolderName)
+				if (folder_name == Queryable.SentMailFolderName && ia.AddressType != GMime.InternetAddressType.Group)
 					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
 			}
 			addrs.Dispose ();
 
 			addrs = GMime.InternetAddressList.ParseString (GMime.Utils.HeaderDecodePhrase (message.Sender));
 			foreach (GMime.InternetAddress ia in addrs) {
-				if (folder_name != Queryable.SentMailFolderName)
+				if (folder_name != Queryable.SentMailFolderName && ia.AddressType != GMime.InternetAddressType.Group)
 					indexable.AddProperty (Property.NewKeyword ("fixme:gotFrom", ia.Addr));
 			}
 			addrs.Dispose ();
 
 			if (folder_name == Queryable.SentMailFolderName)
 				indexable.AddProperty (Property.NewFlag ("fixme:isSent"));
-
+// no need to store date again, use the issent flag to determine if the date is sentdate or not			
+#if false
 			if (folder_name == Queryable.SentMailFolderName)
-				indexable.AddProperty (Property.NewDate ("fixme:sentdate", message.Date));
+				indexable.AddProperty (Property.NewDate ("fixme:sentdate", message.Date.ToUniversalTime ()));
 			else
-				indexable.AddProperty (Property.NewDate ("fixme:received", message.Date));
+				indexable.AddProperty (Property.NewDate ("fixme:received", message.Date.ToUniversalTime ()));
+#endif
 
-			MemoryStream stream = new MemoryStream ();
-			message.WriteToStream (stream);
-			stream.Seek (0, SeekOrigin.Begin);
-			indexable.SetBinaryStream (stream);
+			indexable.SetBinaryStream (message.Stream);
 
 			return indexable;
 		}
